package com.trafficparrot.examples.aws;

import org.apache.camel.*;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.aws.sqs.SqsConstants;
import org.apache.camel.component.mock.MockEndpoint;
import org.apache.camel.impl.JndiRegistry;
import org.apache.camel.test.junit4.CamelTestSupport;
import org.junit.Test;

import static com.trafficparrot.examples.aws.AmazonSQSClient.amazonSQSClient;

public class AmazonSQSApacheCamelIntegrationTest extends CamelTestSupport {

    private static final int VIRTUAL_SERVICE_PORT = 18081;
    private static final String VIRTUAL_SERVICE_URL = "http://localhost:" + VIRTUAL_SERVICE_PORT + "/aws/sqs";
    private static final String SQS_CLIENT_NAME_IN_REGISTRY = "clientName";
    private static final String QUEUE_NAME = "MyQueue";

    @EndpointInject(uri = "direct:start")
    private ProducerTemplate template;

    @EndpointInject(uri = "mock:result")
    private MockEndpoint result;

    @Test
    public void sendIn() throws InterruptedException {
        result.expectedMessageCount(1);

        Exchange exchange = template.send("direct:start", ExchangePattern.InOnly, new Processor() {
            public void process(Exchange exchange) {
                exchange.getIn().setBody("This is my message text.");
            }
        });

        assertMockEndpointsSatisfied();

        Exchange resultExchange = result.getExchanges().get(0);
        assertEquals("This is my message text.", resultExchange.getIn().getBody());
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.MESSAGE_ID));
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.RECEIPT_HANDLE));
        assertEquals("6a1559560f67c5e7a7d5d838bf0272ee", resultExchange.getIn().getHeader(SqsConstants.MD5_OF_BODY));
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.ATTRIBUTES));
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.MESSAGE_ATTRIBUTES));

        assertNotNull(exchange.getIn().getHeader(SqsConstants.MESSAGE_ID));
    }

    @Test
    public void sendInOut() throws Exception {
        result.expectedMessageCount(1);

        Exchange exchange = template.send("direct:start", ExchangePattern.InOut, new Processor() {
            public void process(Exchange exchange) {
                exchange.getIn().setBody("This is my message text.");
            }
        });

        assertMockEndpointsSatisfied();

        Exchange resultExchange = result.getExchanges().get(0);
        assertEquals("This is my message text.", resultExchange.getIn().getBody());
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.RECEIPT_HANDLE));
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.MESSAGE_ID));
        assertEquals("6a1559560f67c5e7a7d5d838bf0272ee", resultExchange.getIn().getHeader(SqsConstants.MD5_OF_BODY));
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.ATTRIBUTES));
        assertNotNull(resultExchange.getIn().getHeader(SqsConstants.MESSAGE_ATTRIBUTES));

        assertNotNull(exchange.getOut().getHeader(SqsConstants.MESSAGE_ID));
        assertEquals("6a1559560f67c5e7a7d5d838bf0272ee", exchange.getOut().getHeader(SqsConstants.MD5_OF_BODY));
    }

    @Override
    protected JndiRegistry createRegistry() throws Exception {
        JndiRegistry registry = super.createRegistry();
        registry.bind(SQS_CLIENT_NAME_IN_REGISTRY, amazonSQSClient(VIRTUAL_SERVICE_URL));
        return registry;
    }

    @Override
    protected RouteBuilder createRouteBuilder() {
        String sqsEndpointUri = "aws-sqs://" + QUEUE_NAME + "?amazonSQSClient=#" + SQS_CLIENT_NAME_IN_REGISTRY;
        return new RouteBuilder() {
            @Override
            public void configure() throws Exception {
                from("direct:start").to(sqsEndpointUri);
                from(sqsEndpointUri).to("mock:result");
            }
        };
    }
}
